@@ -1191,7 +1191,7 @@ proc ApolTop::create { } {
 	    {command "&Domain Transition Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Domain Transition Analysis Help" "dta_help.txt"}}
 	    {command "&Information Flow Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Information Flow Analysis Help" "iflow_help.txt"}}
 	    #{command "&Information Flow Assertion Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Information Flow Assertion Analysis Help" "flow_assertion_help.txt"}}
-	    {command "&File Relabel Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "File Relabel Analysis Help" "file_relabel_help.txt"}}
+	    {command "&Relabel Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Relabel Analysis Help" "file_relabel_help.txt"}}
 	    {command "&Types Relationship Summary Analysis" {all option} "Show help" {} -command {ApolTop::helpDlg "Types Relationship Summary Analysis Help" "types_relation_help.txt"}}
 	    {separator}
 	    {command "&About" {all option} "Show about box" {} -command ApolTop::aboutBox}
@@ -2073,6 +2073,12 @@ proc ApolTop::openPolicyFile {file recent_flag} {
 	if {$rt == 0} {
 		#set filename [file tail $file]
 		set filename $file
+	} elseif {$rt == -6} {
+		tk_messageBox -icon error -type ok -title "Error with policy file" \
+			-message "Pre-version 19 MLS is not supported.\n\n$err" 
+		. configure -cursor $orig_Cursor 
+		focus -force .
+		return -1
 	} else {
 		tk_messageBox -icon error -type ok -title "Error with policy file" \
 			-message "The selected file does not appear to be a valid SE Linux Policy.\n\n$err" 
