@@ -34,7 +34,6 @@
 
 __u32 mls_config = 0;
 
-
 static int skip_ebitmap(ap_fbuf_t *fb, FILE *fp)
 {
 	__u32 *buf, count, highbit, i;
@@ -60,6 +59,39 @@ static int skip_ebitmap(ap_fbuf_t *fb, FILE *fp)
 	return 0;
 }
 
+static int skip_mls_range(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
+{
+	__u32 *buf, nlvl;
+
+	INTERNAL_ASSERTION
+
+	buf = ap_read_fbuf(fb, sizeof(__u32), fp);
+	if(buf == NULL) return fb->err;
+	
+	nlvl = le32_to_cpu(buf[0]);
+	if (fseek(fp, sizeof(__u32)*nlvl, SEEK_CUR))
+		return -3;
+	if (skip_ebitmap(fb,fp))
+		return -3;
+	if (nlvl > 1){
+		if (skip_ebitmap(fb,fp))
+			return -3;
+	} 
+	return 0;
+}
+
+static int skip_mls_level(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
+{
+	__u32 *buf;
+	
+	INTERNAL_ASSERTION
+
+	buf = ap_read_fbuf(fb, sizeof(__u32), fp);
+	if (buf == NULL)
+		return fb->err;
+
+	return skip_ebitmap(fb,fp);
+}
 
 /*********************************************************************
  * NOTE: for all these internal load functions, the following define
@@ -90,11 +122,6 @@ static int load_perm(ap_fbuf_t *fb, FILE *fp, __u32 *val, unsigned int opts, pol
 	len = le32_to_cpu(buf[0]);
 	*val = le32_to_cpu(buf[1]);
 
-	if(mls_config) {
-		buf = ap_read_fbuf(fb, sizeof(__u32), fp); /* mls perm ignore */
-		if(buf == NULL)	return fb->err;
-	}
-	
 	/* perm name */
 	kbuf = ap_read_fbuf(fb, len, fp);
 	if(kbuf == NULL) return fb->err;
@@ -126,7 +153,7 @@ static int load_perms(ap_fbuf_t *fb, FILE *fp, bool_t is_cp, __u32 cval, int cid
 	int i, idx, rt = 0;
 	bool_t keep = FALSE;
 	ap_permission_bmap_t *pmap = NULL;
-	__u32 val, num_cp = 0;
+	__u32 val = 0, num_cp = 0;
 	
 	if(nel == 0)
 		return 0;
@@ -191,7 +218,8 @@ static int load_perms(ap_fbuf_t *fb, FILE *fp, bool_t is_cp, __u32 cval, int cid
 static int load_common_perm(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
 {
 	size_t len, nel;
-	unsigned char *kbuf, *key;
+	unsigned char *kbuf;
+	char *key;
 	__u32 *buf, val;
 	int rt, idx = -1;
 	bool_t keep = FALSE;
@@ -246,8 +274,9 @@ static int load_common_perm(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned in
 
 static int load_class(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
 {
-	size_t len, len2, nel, ncons, nexpr;
-	unsigned char *kbuf, *key, *cbuf;
+	size_t len, len2, nel, ncons, nexpr, nvaltrans;
+	unsigned char *kbuf;
+	char *key, *cbuf;
 	__u32 *buf, expr_type, val, i, j, num_cp = 0, cp_val = -1;
 	int rt, idx = -1, idx2 = -1;		/* idx2 (common perm idx) must ne init'd to -1 for load_perms */
 	bool_t keep = FALSE;
@@ -300,7 +329,7 @@ static int load_class(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts
 		if(cbuf == NULL) return fb->err;
 		if(keep) {
 			cbuf[len2] = '\0';
-			idx2 = get_common_perm_idx(cbuf, policy);
+			idx2 = get_common_perm_idx((const char *)cbuf, policy);
 			if(idx2 < 0) {
 				assert(FALSE); /* debug aide */
 				return -4;
@@ -371,12 +400,45 @@ static int load_class(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts
 		}
 	}
 	
-	/* skip over class MLS info if any */
-	if(mls_config) {
-		if(fseek(fp, sizeof(__u32)*4, SEEK_CUR) != 0)  
-			 return -3;
+	if (policy->version >= POL_VER_19) {
+		buf = ap_read_fbuf(fb, sizeof(__u32), fp);
+		nvaltrans = le32_to_cpu(buf[0]);
+		/* ignore validatetrans constraints */
+		for(i = 0; i < nvaltrans; i++) {
+			buf = ap_read_fbuf(fb, sizeof(__u32)*2, fp); 
+			if(buf == NULL)	return fb->err;
+			/* buf[0] permissions (ignore)
+			 * buf[1] num expressions */
+			nexpr = le32_to_cpu(buf[1]);
+			/* all expressions */
+			for (j = 0; j < nexpr; j++) {
+				buf = ap_read_fbuf(fb, sizeof(__u32)*3, fp); 
+				if(buf == NULL)	return fb->err; 
+				/* buf[0] expression type
+				 * buf[1] attr (ignore)
+				 * buf[2] op (ignore) */
+				expr_type = le32_to_cpu(buf[0]);
+				switch (expr_type) {
+				case CEXPR_NOT:
+				case CEXPR_AND:
+				case CEXPR_OR:
+				case CEXPR_ATTR:
+					/* all of these expression types do not have additional data 
+					 * in file so we can ignore them */
+					break;
+				case CEXPR_NAMES:
+					rt = skip_ebitmap(fb, fp);
+					if(rt != 0)
+						return rt;
+					break;
+				default:
+					return -3;
+					break;
+				}
+			}
+		}
 	}
-	
+
 	if(keep)
 		return idx;
 	else
@@ -387,7 +449,8 @@ static int load_role(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts,
 {
 	__u32 *buf, val;
 	size_t len;
-	unsigned char *kbuf, *key;
+	unsigned char *kbuf;
+	char *key;
 	int idx = -1, rt; 
 	bool_t keep = FALSE;
 
@@ -447,7 +510,8 @@ static int load_type(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts,
 {
 	__u32 *buf, val;
 	size_t len;
-	unsigned char *kbuf, *key;
+	unsigned char *kbuf;
+	char *key;
 	int idx = 0, rt;
 	bool_t keep = FALSE, primary;
 	
@@ -510,7 +574,8 @@ static int load_user(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts,
 {
 	__u32 *buf, val;
 	size_t len;
-	unsigned char *kbuf, *key;
+	unsigned char *kbuf;
+	char *key;
 	bool_t keep = FALSE;
 	int i, rt, idx = -1;
 	ebitmap_t e;
@@ -563,20 +628,11 @@ static int load_user(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts,
 	
 	ebitmap_destroy(&e);
 	
-	/* discard any unsupported MLS stuff */
-	if(mls_config) {
-		__u32 nel, items, i;
-		__u32 *buf;
-		buf = ap_read_fbuf(fb, sizeof(__u32), fp); 
-		nel = le32_to_cpu(buf[0]);
-		for (i = 0; i < nel; i++) {
-			buf = ap_read_fbuf(fb, sizeof(__u32), fp); 
-			items = le32_to_cpu(buf[0]);
-			buf = ap_read_fbuf(fb, sizeof(__u32)*items, fp);
-		}
-
+	if (policy->version >= POL_VER_19) {
+		skip_mls_range(fb, fp, bm, opts, policy);
+		skip_mls_level(fb, fp, bm, opts, policy);
 	}
-	
+
 	return 0;
 }
 
@@ -584,7 +640,7 @@ static int load_bool(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts,
 {
 	__u32 len, *buf, val;
 	bool_t keep = FALSE, state;
-	unsigned char *key;
+	char *key;
 	int idx;
 	
 	INTERNAL_ASSERTION
@@ -1360,9 +1416,7 @@ static int load_cond_list(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int
 	return 0;
 }
 
-
-#ifdef CONFIG_SECURITY_SELINUX_MLS
-static int skip_mls_levels(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
+static int skip_mls_sens(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int opts, policy_t *policy)
 {
 	/* see sens_read() */
 	__u32 *buf, len;
@@ -1374,17 +1428,14 @@ static int skip_mls_levels(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int
 	/* buf[0] len
 	 * buf[1] isalias
 	 */
-	 
+	
 	/* name */
 	len = le32_to_cpu(buf[0]);
-	if(fseek(fp, sizeof(__u32)*len, SEEK_CUR) != 0)  
-			 return -3;
-	
-	/* mls_read_level() */
-	/* sens */
-	if(fseek(fp, sizeof(__u32)*len, SEEK_CUR) != 0)  
+	if(fseek(fp, len, SEEK_CUR) != 0)  
 			 return -3;
 	
+	buf = ap_read_fbuf(fb, sizeof(__u32), fp);
+
 	/* ebitmap */
 	return skip_ebitmap(fb, fp);
 }
@@ -1404,12 +1455,12 @@ static int skip_mls_cats(ap_fbuf_t *fb, FILE *fp, ap_bmaps_t *bm, unsigned int o
 	  */
 	 len = le32_to_cpu(buf[0]);
 	
-	if(fseek(fp, sizeof(__u32)*len, SEEK_CUR) != 0)  
+	if(fseek(fp, len, SEEK_CUR) != 0)  
 			 return -3;
 
 	return 0;
 }
-#endif
+
 
 
 /* main policy load function */
@@ -1424,12 +1475,7 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 	ebitmap_t *e;
 	ap_alias_bmap_t *a;
 
-#ifdef CONFIG_SECURITY_SELINUX_MLS
-	mls_config = POLICYDB_CONFIG_MLS;
-#else
-	mls_config = 0;
-#endif
-	
+
 	if(ap_init_fbuf(&fb) != 0)
 		return -1;
 	bm = ap_new_bmaps();
@@ -1454,25 +1500,32 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 	
 	policy_ver = buf[0];
 	switch(policy_ver) {
+	case POLICYDB_VERSION_MLS:
+		if (set_policy_version(POL_VER_19, policy) != 0) {
+			rt = -4;goto err_return;
+		}
+		mls_config = 1;
+		num_syms = SYM_NUM;
+		break;
 	case POLICYDB_VERSION_NLCLASS:
 		if(set_policy_version(POL_VER_18, policy) != 0)
 			{ rt = -4; goto err_return; }
-		num_syms = SYM_NUM;
+		num_syms = SYM_NUM - 2;
 		break;
 	case POLICYDB_VERSION_IPV6 :
 		if(set_policy_version(POL_VER_17, policy) != 0)
 			{ rt = -4; goto err_return; }
-		num_syms = SYM_NUM;
+		num_syms = SYM_NUM - 2;
 		break;
 	case POLICYDB_VERSION_BOOL:
 		if(set_policy_version(POL_VER_16, policy) != 0)
 			{ rt = -4; goto err_return; }
-		num_syms = SYM_NUM;
+		num_syms = SYM_NUM - 2;
 		break;
 	case POLICYDB_VERSION_BASE:
 		if(set_policy_version(POL_VER_15, policy) != 0)
 			{ rt = -4; goto err_return; }
-		num_syms = SYM_NUM - 1;
+		num_syms = SYM_NUM - 3;
 		break;
 	default: /* unsupported version */
 		return -5;
@@ -1487,11 +1540,12 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 	}
 
 	/* check for MLS stuff and skip over the # of levels */
-	if(buf[1] != mls_config)
-		{ rt = -6; goto err_return; }
-	if(mls_config) {
-		buf = ap_read_fbuf(fb, sizeof(__u32), fp); /* nlevels; ignore */
-		if(buf == NULL) { rt = fb->err; goto err_return; }
+	if(mls_config && buf[1]) {
+		set_policy_version(POL_VER_19MLS, policy);
+	}
+	if(buf[1] && policy->version < POL_VER_MLS) { 
+		rt = -6;
+		goto err_return; 
 	}
 			
 	/* read in symbol tables */
@@ -1508,7 +1562,7 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 				rt = -1;
 				goto err_return;
 			}
-			bm->rev_cp_map = (int *)malloc(sizeof(int) * nprim);
+			bm->rev_cp_map = (__u32 *)malloc(sizeof(__u32) * nprim);
 			if(bm->rev_cp_map == NULL) {
 				rt = -1;
 				goto err_return;
@@ -1575,12 +1629,10 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 			}
 			bm->bool_num = nprim;
 			break;
-#ifdef CONFIG_SECURITY_SELINUX_MLS
-		case 6:		/* MLS levels */
+		case 6:		/* MLS sensitivities */
 		case 7:		/* MLS categories */
 			/* we don't save any MLS stuff yet */
 			break;
-#endif			
 		default:	/* shouldn't get here */
 			rt = -1; 
 			goto err_return;
@@ -1612,16 +1664,14 @@ static int load_binpol(FILE *fp, unsigned int opts, policy_t *policy)
 				rt = load_bool(fb, fp, bm, opts, policy);
 				if(rt < 0 && rt != LOAD_SUCCESS_NO_SAVE) goto err_return;
 				break;
-#ifdef CONFIG_SECURITY_SELINUX_MLS
 			case 6:		/* MLS levels */
-				rt = skip_mls_levels(fb, fp, bm, opts, policy);
+				rt = skip_mls_sens(fb, fp, bm, opts, policy);
 				if(rt < 0) return rt;
 				break;
 			case 7:		/* MLS categories */
 				rt = skip_mls_cats(fb, fp, bm, opts, policy);
 				if(rt < 0) goto err_return;
 				break;
-#endif			
 			default:	/* shouldn't get here */
 				{ rt = -1; goto err_return; };
 				break;
