@@ -320,18 +320,15 @@ static int parse_context(char *token, char *user, char *role, char *type)
 	/* Parse user:role:type */
 	int i = 0;
 	char *fields[PARSE_NUM_CONTEXT_FIELDS];
-
 	assert(token != NULL);
-        while ((fields[i] = strsep(&token, ":")) != NULL && i < PARSE_NUM_CONTEXT_FIELDS) {
-		i++;       	       	
-        }
-
+	while (i < PARSE_NUM_CONTEXT_FIELDS && (fields[i] = strsep(&token,":")) != NULL){
+		i++;
+	}
 	if (i != PARSE_NUM_CONTEXT_FIELDS)
 		return -1;
-
-	strcpy(user, fields[0]);
-	strcpy(role, fields[1]);
-	strcpy(type, fields[2]);
+	strcpy(user,fields[0]);
+	strcpy(role,fields[1]);
+	strcpy(type,fields[2]);
 	return PARSE_RET_SUCCESS;
        
 }
@@ -355,7 +352,7 @@ static int avc_msg_insert_tcon(msg_t *msg, char **tmp, audit_log_t *log)
 			free(role);
 			return MSG_MEMORY_ERROR;
 		}
-		if (parse_context(*tmp, user, role, type) < 0){
+		if (parse_context(*tmp, user, role, type) != PARSE_RET_SUCCESS){
 			free(user);
 			free(role);
 			free(type);
@@ -415,7 +412,7 @@ static int avc_msg_insert_scon(msg_t *msg, char **tmp, audit_log_t *log)
 			return MSG_MEMORY_ERROR;
 		}
 
-		if (parse_context(*tmp, user, role, type) < 0) {
+		if (parse_context(*tmp, user, role, type) != PARSE_RET_SUCCESS) {
 			free(user);
 			free(role);
 			free(type);
@@ -1108,7 +1105,7 @@ static unsigned int boolean_msg_insert_field_data(char **tokens, msg_t **msg, au
         int i, num_bools = 0, num_bools_valid = 0, bool, start_bools_pos;
 	int *booleans = NULL, position = 0, bool_idx = 0;
 	unsigned int ret = 0, tmp_ret = 0;
-	bool_t *values = NULL, val;
+	bool_t *values = NULL, val = FALSE;
 	
 	assert(tokens != NULL && msg != NULL && *msg != NULL && log != NULL && num_tokens > 0);
 
