@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Tresys Technology, LLC
+/* Copyright (C) 2003-2005 Tresys Technology, LLC
  * see file 'COPYING' for use and warranty information */
 
 /*
@@ -221,7 +221,6 @@ static int do_policy_query(GString *src_type, GString *tgt_type, GString *obj_cl
 	init_teq_query(&q);
 	q.rule_select = TEQ_ALLOW;
 	q.any = FALSE;
-
 	widget = glade_xml_get_widget(xml, "SrcTypeIndirectCheck");
 	g_assert(widget);
 	q.ta1.indirect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -409,7 +408,7 @@ static void query_window_populate_combo_boxes(GtkWidget *src_type_combo, GtkWidg
 {
 	GList *items = NULL;
 	int i;
-	
+
 	for (i = 0; i < seaudit_app->cur_policy->num_types; i++) {
 		items = g_list_append(items, seaudit_app->cur_policy->types[i].name);
 	}
@@ -428,7 +427,7 @@ static void query_window_populate_combo_boxes(GtkWidget *src_type_combo, GtkWidg
 	return;
 }
 
-static void populate_query_window_widgets(GladeXML *xml)
+static void populate_query_window_widgets(GladeXML *xml, int *tree_item_idx)
 {
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
@@ -441,23 +440,12 @@ static void populate_query_window_widgets(GladeXML *xml)
 	avc_msg_t *avc_msg = NULL;
 	int fltr_msg_idx, msg_list_idx;
 	seaudit_filtered_view_t *view;
-
+	GtkTreePath *path = NULL;
+	GList *glist = NULL, *item = NULL;
+	
 	g_assert(seaudit_app->cur_policy);
 	view = seaudit_window_get_current_view(seaudit_app->window);
-	sel = gtk_tree_view_get_selection(view->tree_view);
-	selected = gtk_tree_selection_get_selected(sel, &model, &iter);
-	if (selected) {
-		fltr_msg_idx = seaudit_log_view_store_iter_to_idx((SEAuditLogViewStore*)model, &iter);
-		msg_list_idx = view->store->log_view->fltr_msgs[fltr_msg_idx];
-		msg = seaudit_app->cur_log->msg_list[msg_list_idx];
-		if (msg->msg_type!=AVC_MSG) {
-			selected = FALSE;
-		} else {
-			g_assert(fltr_msg_idx >= 0);
-			avc_msg = msg->msg_data.avc_msg;
-		}
-	}
-
+	
 	src_type_combo = glade_xml_get_widget(xml, "src_combo");
 	g_assert(src_type_combo);
 	tgt_type_combo = glade_xml_get_widget(xml, "tgt_combo");
@@ -468,16 +456,53 @@ static void populate_query_window_widgets(GladeXML *xml)
 	gtk_combo_disable_activate(GTK_COMBO(src_type_combo));   
 	gtk_combo_disable_activate(GTK_COMBO(tgt_type_combo));   
 	gtk_combo_disable_activate(GTK_COMBO(obj_class_combo));   
-
+	
 	query_window_populate_combo_boxes(src_type_combo, tgt_type_combo, obj_class_combo);
-
+	
 	src_entry = glade_xml_get_widget(xml, "src_combo_entry");
 	g_assert(src_entry);	
 	tgt_entry = glade_xml_get_widget(xml, "tgt_combo_entry");
 	g_assert(tgt_entry);
 	obj_entry = glade_xml_get_widget(xml, "obj_combo_entry");
 	g_assert(obj_entry);	
-
+	
+	gtk_entry_set_text(GTK_ENTRY(src_entry), "");
+	gtk_entry_set_text(GTK_ENTRY(tgt_entry), "");
+	gtk_entry_set_text(GTK_ENTRY(obj_entry), "");
+	
+	if (tree_item_idx == NULL) {
+		sel = gtk_tree_view_get_selection(view->tree_view);
+		glist = gtk_tree_selection_get_selected_rows(sel, &model);
+		if (glist == NULL) {
+			return;
+		}
+		/* Only grab the top-most selected item */
+		item = glist;
+		path = item->data;
+		if (gtk_tree_model_get_iter(model, &iter, path) == 0) {
+			fprintf(stderr, "Could not get valid iterator for the selected path.\n");
+			if (glist) {	
+				g_list_foreach(glist, (GFunc) gtk_tree_path_free, NULL);
+				g_list_free (glist);			
+			}
+			return;	
+		}
+		fltr_msg_idx = seaudit_log_view_store_iter_to_idx((SEAuditLogViewStore*)model, &iter);
+		selected = TRUE;
+	} else {
+		fltr_msg_idx = *tree_item_idx;	
+		selected = TRUE;
+	}
+	
+	msg_list_idx = view->store->log_view->fltr_msgs[fltr_msg_idx];
+	msg = seaudit_app->cur_log->msg_list[msg_list_idx];
+	if (msg->msg_type!=AVC_MSG) {
+		selected = FALSE;
+	} else {
+		g_assert(fltr_msg_idx >= 0);
+		avc_msg = msg->msg_data.avc_msg;
+	}
+	
 	if (selected) {
 		str = g_string_new("");
 		g_string_assign(str, audit_log_get_type(seaudit_app->cur_log, avc_msg->src_type));
@@ -490,11 +515,14 @@ static void populate_query_window_widgets(GladeXML *xml)
 		gtk_entry_set_text(GTK_ENTRY(tgt_entry), str->str);
 		gtk_entry_set_text(GTK_ENTRY(obj_entry), audit_log_get_obj(seaudit_app->cur_log, avc_msg->obj_class));
 		g_string_free(str, TRUE);
-	} else { 
-		gtk_entry_set_text(GTK_ENTRY(src_entry), "");
-		gtk_entry_set_text(GTK_ENTRY(tgt_entry), "");
-		gtk_entry_set_text(GTK_ENTRY(obj_entry), "");
-	}
+		
+		/* Free selected rows list */
+		if (glist) {
+			g_list_foreach(glist, (GFunc)gtk_tree_path_free, NULL);
+			g_list_free(glist);
+		}
+	} 
+		
 	return;
 }
 
@@ -545,7 +573,7 @@ void on_event_after(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 	return;
 }
 
-int query_window_create(void)
+int query_window_create(int *tree_item_idx)
 {
 	GladeXML *xml;
 	GtkWidget *text, *button, *combo;
@@ -589,7 +617,7 @@ int query_window_create(void)
 
 	policy_load_callback_register(&on_new_policy_opened, window);
 	
-	populate_query_window_widgets(xml);
+	populate_query_window_widgets(xml, tree_item_idx);
 	
 	if (is_binary_policy(seaudit_app->cur_policy)) {
 		/* Remove the policy.conf tab if this is a binary policy. */
