@@ -71,7 +71,7 @@ static int check_str_sz(char **str, int *sz, int needed)
  * 1	provided user does not exists
  * -1	other error
  */
-const int seu_show_users(const char *user, char **outstr, user_db_t *db, policy_t *policy)
+int seu_show_users(const char *user, char **outstr, user_db_t *db, policy_t *policy)
 {
 	char *name = NULL, *role_name = NULL, *tmp = NULL;
 	int sz, i, j, rt;
@@ -152,7 +152,7 @@ err:
 }
 
 /* display all roles */
-const int seu_show_roles(char **outstr, user_db_t *db, policy_t *policy)
+int seu_show_roles(char **outstr, user_db_t *db, policy_t *policy)
 {
 	char *tmp, *role;
 	int i, sz;
@@ -325,6 +325,7 @@ int main(int argc, char *argv[])
 		/* replaced -g with less desireable -X because -g already used by
 		* one of the user[add|mod|del] commands */
 		if(argv[1][1] == 'X' ) { 
+			snprintf(prog_path, PATH_MAX, "./%s", SEUSER_GUI_PROG);
 			rt = access(prog_path, X_OK);
 			if (rt == 0) {
 				rt = execvp(prog_path, argv);
